import 'dart:async';

import 'package:flutterx_http/flutterx_http.dart';
import 'package:http/http.dart';

class TransformRequest extends BaseRequest implements CloneableRequest {
  final BaseRequest source;
  final StreamTransformer<List<int>, List<int>> transform;

  @override
  int? get contentLength => source.contentLength;

  @override
  set contentLength(int? value) => throw UnsupportedError('cannot change TransformRequest configuration');

  @override
  bool get persistentConnection => source.persistentConnection;

  @override
  set persistentConnection(bool value) => throw UnsupportedError('cannot change TransformRequest configuration');

  @override
  bool get followRedirects => source.followRedirects;

  @override
  set followRedirects(bool value) => throw UnsupportedError('cannot change TransformRequest configuration');

  @override
  int get maxRedirects => source.maxRedirects;

  @override
  set maxRedirects(int value) => throw UnsupportedError('cannot change TransformRequest configuration');

  @override
  Map<String, String> get headers => source.headers;

  TransformRequest({required this.source, required this.transform}) : super(source.method, source.url);

  @override
  ByteStream finalize() {
    super.finalize();
    return ByteStream(source.finalize().transform(transform));
  }

  @override
  TransformRequest copy({
    BaseRequest? source,
    StreamTransformer<List<int>, List<int>>? transform,
    String? method,
    Uri? url,
    bool? persistentConnection,
    bool? followRedirects,
    int? maxRedirects,
    Map<String, String>? headers,
  }) =>
      TransformRequest(
          source: (source ?? this.source).copy(
              method: method ?? this.source.method,
              url: url ?? this.source.url,
              persistentConnection: persistentConnection ?? this.source.persistentConnection,
              followRedirects: followRedirects ?? this.source.followRedirects,
              maxRedirects: maxRedirects ?? this.source.maxRedirects,
              headers: headers ?? this.source.headers),
          transform: transform ?? this.transform);
}
